#!/usr/bin/perl

use strict;
use warnings;

require CGI;
use CGI::Carp qw(fatalsToBrowser);

require LWP::Simple;
require XML::LibXML;
require Template;

my $phonelist_url   = 'http://intranet.example.com/phonelist';
my $mirror_filename = '/var/tmp/phonelist.html';

send_phonelist_ldif();

exit;

sub send_phonelist_ldif {
  my $cgi = CGI->new;

  my $template = Template->new({
    INCLUDE_PATH => '/home/grant/projects/talks/phonelist/cgi-bin',
    TAG_STYLE    => 'asp',
  });

  print $cgi->header(-type => 'text/plain');

  my $data = { people => get_phonelist_data() };

  $template->process('phonelist.ldif', $data);
}

sub get_phonelist_data {
  my $dom = get_phonelist_html();

  my @col_names = qw(first_name surname ext ddi mobile email);

  my @results;
  foreach my $row ($dom->findnodes('//tr')) {
    my @f = map {
      $_ = $_->to_literal ;
      s/\x{A0}/ /sg;              # &nbsp; => space
      s/^\s+//g;                  # strip leading space
      s/\s+$//g;                  # strip trailing space
      s/\s\s+/ /g;                # collapse spaces
      $_;
    } $row->findnodes('./td');
    next if(@f != @col_names  or  $f[-1] !~ /@/);
    my %rec;
    @rec{@col_names} = @f;
    format_phone($rec{$_}) foreach(qw(ext ddi mobile));
    ($rec{uid}) = $rec{email} =~ /(\S+)@/;
    push @results, \%rec;
  }

  return \@results;
}

sub format_phone {
  $_[0] =~ s/\D//g;                 # strip non-digits
  $_[0] =~ s/^(02\d)/$1 /;          # space after 021, 027 etc
  $_[0] =~ s/(\d{4})(\d{4})/$1 $2/; # 8 digits as: nnnn nnnn
  $_[0] =~ s/(\d{3})(\d{4})/$1 $2/; # 7 digits as: nnn nnnn
  $_[0] =~ s/(\d{3})(\d{3})/$1 $2/; # 6 digits as: nnn nnn
}

sub get_phonelist_html {
  LWP::Simple::mirror($phonelist_url, $mirror_filename);

  local(*STDERR);
  open STDERR, ">/dev/null";
  my $parser = XML::LibXML->new();
  $parser->recover(1);
  return $parser->parse_html_file($mirror_filename);
}

