#!/usr/bin/perl

use strict;
use warnings;

require CGI;
use CGI::Carp qw(fatalsToBrowser);

require LWP::Simple;
require XML::LibXML;
require PDF::Report;

my $phonelist_url   = 'http://intranet.example.com/phonelist';
my $mirror_filename = '/var/tmp/phonelist.html';

# PDF Offsets

my $olh = 17;            # Line height
my $obm = 33;            # Bottom margin
my $otm = 464;           # Top margin (below header)
my $orm = 810;           # Right margin

my $ofn = 34;            # First name
my $osn = $ofn + 50;     # Surname
my $oxt = $osn + 50;     # Extension
my $odd = $oxt + 26;     # DDI
my $omb = $odd + 54;     # Mobile number
my $oem = $omb + 74;     # Email
my $oc2 = 403;           # Column two
my $osl = $ofn - 6;      # Shading left
my $osr = $orm + 6;      # Shading right

send_phonelist_pdf();

exit;

sub send_phonelist_pdf {
  my $cgi = CGI->new;

  my $buf = make_phonelist_pdf();

  print $cgi->header(
          -type => 'application/pdf', -content_length => length($buf)
        ), $buf;
}

sub make_phonelist_pdf {
  my $data = get_phonelist_data();
  
  my $pdf = PDF::Report->new(
    PageSize        => "A4", 
    PageOrientation => "Landscape"
  );
  $pdf->newpage(1);  # Turn off auto page numbering
  $pdf->openpage;

  add_pdf_header($pdf);
  add_pdf_column_headings($pdf);
  add_pdf_bars($pdf);
  add_pdf_listings($pdf, $data);

  return $pdf->Finish;
}

sub add_pdf_header {
  my($pdf) = @_;

  $pdf->setSize(18);
  $pdf->setFont('Helvetica-Bold');
  $pdf->centerString(0, 845, 550, 'Example Corp - Phone List');

  $pdf->setSize(11);
  $pdf->setFont('Helvetica');
  $pdf->addRawText('Level 27, 150 Willis Street',    274, 527);
  $pdf->addRawText('PO Box 55-502, Manners Street',  274, 514);
  $pdf->addRawText('Telephone:',                     462, 527);
  $pdf->addRawText('Facsimilie:',                    462, 514);
  $pdf->addRawText('04 555 2267',                    519, 527);
  $pdf->addRawText('04 555 5596',                    519, 514);
}

sub add_pdf_column_headings {
  my($pdf) = @_;

  $pdf->setSize(11);
  $pdf->setFont('Helvetica-Bold');
  my $y = $otm + $olh;
  foreach my $x (0, $oc2) {
    $pdf->addRawText('Name',   $x + $ofn, $y);
    $pdf->addRawText('Ext',    $x + $oxt, $y);
    $pdf->addRawText('DDI',    $x + $odd + 10, $y);
    $pdf->addRawText('Mobile', $x + $omb + 10, $y);
    $pdf->addRawText('Email',  $x + $oem + 26, $y);
  }
}

sub add_pdf_bars {
  my($pdf) = @_;

  my $row_y = $otm;
  while($row_y > $obm) {
    my $row_t = $row_y + $olh - 5;
    my $row_b = $row_y - 5;
    $pdf->shadeRect($osl, $row_t, $osr, $row_b, '#eeeeee');
    $row_y -= $olh * 2;
  }

}

sub add_pdf_listings {
  my($pdf, $data) = @_;

  $pdf->setSize(10.0);
  $pdf->setFont('Helvetica');
  
  my $lines = int(($otm - $obm) / $olh + 0.5);
  my $count = @$data;

  $pdf->addRawText(' ', 20, 20);

  foreach my $i (0..$count-1) {
    my $col = int($i / $lines);
    my $row = $i % $lines;

    add_pdf_entry($pdf, $row, $col, $data->[$i]);
  }
}

sub add_pdf_entry {
  my($pdf, $row, $col, $data) = @_;

  my $x = $oc2 * $col;
  my $y = $otm - $olh * $row;

  my $name = "$data->{first_name} $data->{surname}";

  $pdf->addRawText($name,           $x + $ofn, $y);
  $pdf->addRawText($data->{ext},    $x + $oxt, $y);
  $pdf->addRawText($data->{ddi},    $x + $odd, $y);
  $pdf->addRawText($data->{mobile}, $x + $omb, $y);
  $pdf->addRawText($data->{email},  $x + $oem, $y);
}

sub get_phonelist_data {
  my $dom = get_phonelist_html();

  my @col_names = qw(first_name surname ext ddi mobile email);

  my @results;
  foreach my $row ($dom->findnodes('//tr')) {
    my @f = map {
      $_ = $_->to_literal ;
      s/\x{A0}/ /sg;              # &nbsp; => space
      s/^\s+//g;                  # strip leading space
      s/\s+$//g;                  # strip trailing space
      s/\s\s+/ /g;                # collapse spaces
      $_;
    } $row->findnodes('./td');
    next if(@f != @col_names  or  $f[-1] !~ /@/);
    my %rec;
    @rec{@col_names} = @f;
    format_phone($rec{$_}) foreach(qw(ext ddi mobile));
    push @results, \%rec;
  }

  return \@results;
}

sub format_phone {
  $_[0] =~ s/\D//g;                 # strip non-digits
  $_[0] =~ s/^(02\d)/$1 /;          # space after 021, 027 etc
  $_[0] =~ s/(\d{4})(\d{4})/$1 $2/; # 8 digits as: nnnn nnnn
  $_[0] =~ s/(\d{3})(\d{4})/$1 $2/; # 7 digits as: nnn nnnn
  $_[0] =~ s/(\d{3})(\d{3})/$1 $2/; # 6 digits as: nnn nnn
}

sub get_phonelist_html {
  LWP::Simple::mirror($phonelist_url, $mirror_filename);

  local(*STDERR);
  open STDERR, ">/dev/null";
  my $parser = XML::LibXML->new();
  $parser->recover(1);
  return $parser->parse_html_file($mirror_filename);
}

